@@ -474,12 +474,13 @@ class TargetAndroid(Target):
                 ext = 'tar.bz2'
             else:
                 ext = 'zip'
-            archive = 'android-ndk-r{0}-' + _platform + '-{1}.' + ext
+            archive = 'android-ndk-r{0}-' + _platform + '{1}.' + ext
             is_64 = (os.uname()[4] == 'x86_64')
         else:
             raise SystemError('Unsupported platform: {}'.format(platform))
 
         architecture = 'x86_64' if is_64 else 'x86'
+        architecture = '' if _version >= 23 else f'-{architecture}'
         unpacked = 'android-ndk-r{0}'
         archive = archive.format(self.android_ndk_version, architecture)
         unpacked = unpacked.format(self.android_ndk_version)
@@ -917,6 +918,12 @@ class TargetAndroid(Target):
             cmd.append('--blacklist')
             cmd.append(realpath(expanduser(blacklist_src)))
 
+        # support for java directory
+        javadirs = self.buildozer.config.getlist('app', 'android.add_src', [])
+        for javadir in javadirs:
+            cmd.append('--add-source')
+            cmd.append(realpath(expanduser(javadir)))
+
         # support for aars
         aars = self.buildozer.config.getlist('app', 'android.add_aars', [])
         for aar in aars:
@@ -991,7 +998,7 @@ class TargetAndroid(Target):
 
     def get_release_mode(self):
         # aab, also if unsigned is named as *-release
-        if self.check_p4a_sign_env() or self.artifact_format == "aab":
+        if self.check_p4a_sign_env() or self.artifact_format in ["aab", "aar"]:
             return "release"
         return "release-unsigned"
 
@@ -1111,9 +1118,6 @@ class TargetAndroid(Target):
         # update the project.properties libraries references
         self._update_libraries_references(dist_dir)
 
-        # add src files
-        self._add_java_src(dist_dir)
-
         # generate the whitelist if needed
         self._generate_whitelist(dist_dir)
 
@@ -1321,6 +1325,9 @@ class TargetAndroid(Target):
                 artifact_dir = join(dist_dir, "build", "outputs", "apk", mode_sign)
             elif self.artifact_format == "aab":
                 artifact_dir = join(dist_dir, "build", "outputs", "bundle", mode_sign)
+            elif self.artifact_format == "aar":
+                artifact_dir = join(dist_dir, "build", "outputs", "aar")
+
         else:
             # on ant, the apk use the title, and have version
             bl = u'\'" ,'
@@ -1398,26 +1405,6 @@ class TargetAndroid(Target):
 
         self.buildozer.debug('project.properties updated')
 
-    def _add_java_src(self, dist_dir):
-        java_src = self.buildozer.config.getlist('app', 'android.add_src', [])
-
-        gradle_files = ["build.gradle", "gradle", "gradlew"]
-        is_gradle_build = any((
-            exists(join(dist_dir, x)) for x in gradle_files))
-        if is_gradle_build:
-            src_dir = join(dist_dir, "src", "main", "java")
-            self.buildozer.info(
-                "Gradle project detected, copy files {}".format(src_dir))
-        else:
-            src_dir = join(dist_dir, 'src')
-            self.buildozer.info(
-                "Ant project detected, copy files in {}".format(src_dir))
-
-        for pattern in java_src:
-            for fn in glob(expanduser(pattern.strip())):
-                last_component = basename(fn)
-                self.buildozer.file_copytree(fn, join(src_dir, last_component))
-
     @property
     def serials(self):
         if hasattr(self, '_serials'):
