@@ -3,9 +3,6 @@ iOS target, based on kivy-ios project
 '''
 
 import sys
-if sys.platform != 'darwin':
-    raise NotImplementedError('Windows platform not yet working for Android')
-
 import plistlib
 from buildozer import BuildozerCommandException
 from buildozer.target import Target, no_config
@@ -63,7 +60,18 @@ li { padding: 1em; }
 class TargetIos(Target):
     targetname = "ios"
 
+    def __init__(self, buildozer):
+        super().__init__(buildozer)
+        executable = sys.executable or 'python'
+        self._toolchain_cmd = f"{executable} toolchain.py "
+        self._xcodebuild_cmd = "xcodebuild "
+        # set via install_platform()
+        self.ios_dir = None
+        self.ios_deploy_dir = None
+
     def check_requirements(self):
+        if sys.platform != "darwin":
+            raise NotImplementedError("Only macOS is supported for iOS target")
         checkbin = self.buildozer.checkbin
         cmd = self.buildozer.cmd
 
@@ -93,18 +101,46 @@ class TargetIos(Target):
         self.buildozer.debug(' -> found {0}'.format(xcode))
 
     def install_platform(self):
+        """
+        Clones `kivy/kivy-ios` and `phonegap/ios-deploy` then sets `ios_dir`
+        and `ios_deploy_dir` accordingly.
+        """
         self.ios_dir = self.install_or_update_repo('kivy-ios', platform='ios')
         self.ios_deploy_dir = self.install_or_update_repo('ios-deploy',
                                                           platform='ios',
                                                           branch='1.7.0',
                                                           owner='phonegap')
 
+    def toolchain(self, cmd, **kwargs):
+        kwargs.setdefault('cwd', self.ios_dir)
+        return self.buildozer.cmd(self._toolchain_cmd + cmd, **kwargs)
+
+    def xcodebuild(self, *args, **kwargs):
+        return self.buildozer.cmd(self._xcodebuild_cmd + ' '.join(arg for arg in args if arg is not None), **kwargs)
+
+    @property
+    def code_signing_allowed(self):
+        allowed = self.buildozer.config.getboolean("app", "ios.codesign.allowed")
+        allowed = "YES" if allowed else "NO"
+        return f"CODE_SIGNING_ALLOWED={allowed}"
+
+    @property
+    def code_signing_development_team(self):
+        team = self.buildozer.config.getdefault("app", f"ios.codesign.development_team.{self.build_mode}", None)
+        return f"DEVELOPMENT_TEAM={team}" if team else None
+
     def get_available_packages(self):
-        available_modules = self.buildozer.cmd(
-                './toolchain.py recipes --compact',
-                cwd=self.ios_dir, get_stdout=True)[0]
+        available_modules = self.toolchain("recipes --compact", get_stdout=True)[0]
         return available_modules.splitlines()[0].split()
 
+    def load_plist_from_file(self, plist_rfn):
+        with open(plist_rfn, 'rb') as f:
+            return plistlib.load(f)
+
+    def dump_plist_to_file(self, plist, plist_rfn):
+        with open(plist_rfn, 'wb') as f:
+            plistlib.dump(plist, f)
+
     def compile_platform(self):
         # for ios, the compilation depends really on the app requirements.
         # compile the distribution only if the requirements changed.
@@ -139,11 +175,10 @@ class TargetIos(Target):
             return
 
         modules_str = ' '.join(ios_requirements)
-        self.buildozer.cmd('./toolchain.py build {}'.format(modules_str),
-                           cwd=self.ios_dir)
+        self.toolchain(f"build {modules_str}")
 
         if not self.buildozer.file_exists(self.ios_deploy_dir, 'ios-deploy'):
-            self.buildozer.cmd('make ios-deploy', cwd=self.ios_deploy_dir)
+            self.xcodebuild(cwd=self.ios_deploy_dir)
 
         self.buildozer.state['ios.requirements'] = ios_requirements
         self.buildozer.state.sync()
@@ -170,19 +205,17 @@ class TargetIos(Target):
 
         self.app_project_dir = join(self.ios_dir, '{0}-ios'.format(app_name.lower()))
         if not self.buildozer.file_exists(self.app_project_dir):
-            create_cmd = './toolchain.py create {0}{1} {2}'.format(frameworks_cmd, app_name,
-                                                                   self.buildozer.app_dir)
-            self.buildozer.cmd(create_cmd, cwd=self.ios_dir)
+            cmd = f"create {frameworks_cmd}{app_name} {self.buildozer.app_dir}"
         else:
-            update_cmd = './toolchain.py update {0}{1}-ios'.format(frameworks_cmd, app_name)
-            self.buildozer.cmd(update_cmd, cwd=self.ios_dir)
+            cmd = f"update {frameworks_cmd}{app_name}-ios"
+        self.toolchain(cmd)
 
         # fix the plist
         plist_fn = '{}-Info.plist'.format(app_name.lower())
         plist_rfn = join(self.app_project_dir, plist_fn)
         version = self.buildozer.get_version()
         self.buildozer.info('Update Plist {}'.format(plist_fn))
-        plist = plistlib.readPlist(plist_rfn)
+        plist = self.load_plist_from_file(plist_rfn)
         plist['CFBundleIdentifier'] = self._get_package()
         plist['CFBundleShortVersionString'] = version
         plist['CFBundleVersion'] = '{}.{}'.format(version,
@@ -191,25 +224,40 @@ class TargetIos(Target):
         # add icons
         self._create_icons()
 
-        # ok, write the modified plist.
-        plistlib.writePlist(plist, plist_rfn)
+        # Generate OTA distribution manifest if `app_url`, `display_image_url` and `full_size_image_url` are defined.
+        app_url = self.buildozer.config.getdefault("app", "ios.manifest.app_url", None)
+        display_image_url = self.buildozer.config.getdefault("app", "ios.manifest.display_image_url", None)
+        full_size_image_url = self.buildozer.config.getdefault("app", "ios.manifest.full_size_image_url", None)
 
-        mode = 'Debug' if self.build_mode == 'debug' else 'Release'
-        self.buildozer.cmd('xcodebuild -configuration {} ENABLE_BITCODE=NO clean build'.format(mode),
-                cwd=self.app_project_dir)
+        if any((app_url, display_image_url, full_size_image_url)):
+
+            if not all((app_url, display_image_url, full_size_image_url)):
+                self.buildozer.error("Options ios.manifest.app_url, ios.manifest.display_image_url"
+                                     " and ios.manifest.full_size_image_url should be defined all together")
+                return
+
+            plist['manifest'] = {
+                'appURL': app_url,
+                'displayImageURL': display_image_url,
+                'fullSizeImageURL': full_size_image_url,
+            }
+
+        # ok, write the modified plist.
+        self.dump_plist_to_file(plist, plist_rfn)
+
+        mode = self.build_mode.capitalize()
+        self.xcodebuild(
+            f'-configuration {mode}',
+            '-allowProvisioningUpdates',
+            'ENABLE_BITCODE=NO',
+            self.code_signing_allowed,
+            self.code_signing_development_team,
+            'clean build',
+            cwd=self.app_project_dir)
         ios_app_dir = '{app_lower}-ios/build/{mode}-iphoneos/{app_lower}.app'.format(
                 app_lower=app_name.lower(), mode=mode)
         self.buildozer.state['ios:latestappdir'] = ios_app_dir
 
-        key = 'ios.codesign.{}'.format(self.build_mode)
-        ioscodesign = self.buildozer.config.getdefault('app', key, '')
-        if not ioscodesign:
-            self.buildozer.error('Cannot create the IPA package without'
-                ' signature. You must fill the "{}" token.'.format(key))
-            return
-        elif ioscodesign[0] not in ('"', "'"):
-            ioscodesign = '"{}"'.format(ioscodesign)
-
         intermediate_dir = join(self.ios_dir, '{}-{}.intermediates'.format(app_name, version))
         xcarchive = join(intermediate_dir, '{}-{}.xcarchive'.format(
             app_name, version))
@@ -221,27 +269,35 @@ class TargetIos(Target):
         self.buildozer.rmdir(intermediate_dir)
 
         self.buildozer.info('Creating archive...')
-        self.buildozer.cmd((
-                '/usr/bin/xcodebuild'
-                ' -alltargets'
-                ' -configuration {mode}'
-                ' -scheme {scheme}'
-                ' -archivePath "{xcarchive}"'
-                ' archive'
-                ' ENABLE_BITCODE=NO'
-            ).format(mode=mode, xcarchive=xcarchive, scheme=app_name.lower()),
+        self.xcodebuild(
+            '-alltargets',
+            f'-configuration {mode}',
+            f'-scheme {app_name.lower()}',
+            f'-archivePath "{xcarchive}"',
+            '-destination \'generic/platform=iOS\'',
+            'archive',
+            'ENABLE_BITCODE=NO',
+            self.code_signing_allowed,
+            self.code_signing_development_team,
             cwd=build_dir)
 
+        key = 'ios.codesign.{}'.format(self.build_mode)
+        ioscodesign = self.buildozer.config.getdefault('app', key, '')
+        if not ioscodesign:
+            self.buildozer.error('Cannot create the IPA package without'
+                ' signature. You must fill the "{}" token.'.format(key))
+            return
+        elif ioscodesign[0] not in ('"', "'"):
+            ioscodesign = '"{}"'.format(ioscodesign)
+
         self.buildozer.info('Creating IPA...')
-        self.buildozer.cmd((
-                '/usr/bin/xcodebuild'
-                ' -exportArchive'
-                ' -exportFormat IPA'
-                ' -archivePath "{xcarchive}"'
-                ' -exportPath "{ipa}"'
-                ' CODE_SIGN_IDENTITY={ioscodesign}'
-                ' ENABLE_BITCODE=NO'
-            ).format(xcarchive=xcarchive, ipa=ipa_tmp, ioscodesign=ioscodesign),
+        self.xcodebuild(
+            '-exportArchive',
+            f'-archivePath "{xcarchive}"',
+            f'-exportOptionsPlist "{plist_rfn}"',
+            f'-exportPath "{ipa_tmp}"',
+            f'CODE_SIGN_IDENTITY={ioscodesign}',
+            'ENABLE_BITCODE=NO',
             cwd=build_dir)
 
         self.buildozer.info('Moving IPA to bin...')
@@ -301,13 +357,13 @@ class TargetIos(Target):
             self.buildozer.error('Icon {} does not exists'.format(icon_fn))
             return
 
-        self.buildozer.cmd('./toolchain.py icon {} {}'.format(
-            self.app_project_dir, icon_fn),
-            cwd=self.ios_dir)
+        self.toolchain(f"icon {self.app_project_dir} {icon_fn}")
 
     def check_configuration_tokens(self):
         errors = []
         config = self.buildozer.config
+        if not config.getboolean('app', 'ios.codesign.allowed'):
+            return
         identity_debug = config.getdefault('app', 'ios.codesign.debug', '')
         identity_release = config.getdefault('app', 'ios.codesign.release',
                 identity_debug)
@@ -376,10 +432,10 @@ class TargetIos(Target):
             if not error:
                 correct = True
                 break
-            self.error('Invalid keychain password')
+            self.buildozer.error('Invalid keychain password')
 
         if not correct:
-            self.error('Unable to unlock the keychain, exiting.')
+            self.buildozer.error('Unable to unlock the keychain, exiting.')
             raise BuildozerCommandException()
 
         # maybe user want to save it for further reuse?
